@@ -2355,7 +2355,12 @@ gok_key_isRepeatable(GokKey* pKey)
 	return returncode;
 }
 
-
+/* 
+ * the reason for this static: GokOutput is not a GObject, 
+ * so we have no ref counting ability.
+ * therefore we only delete a wordcomplete output when creating a new one. 
+ * It would be better to do proper ref counting.
+ */
 static GokOutput *last_wordcomplete_output = NULL;
 
 /**
@@ -2390,22 +2395,24 @@ gok_key_wordcomplete_output (GokKey *pKey, GokWordComplete *complete)
 	
 	if (word_part && (g_utf8_strlen (word_part, -1) < g_utf8_strlen (LabelText, -1)))
 	{
-		gchar *cp;
-                /* 
-		 * note that this won't match if the decompositions don't match. Should we
-		 * decompose to canonical first ? 
-		 */ 
-		cp = strstr (LabelText, word_part); 
+		gchar *cp, *word_part_canonical;
+
+		if (last_wordcomplete_output) {
+		    gok_output_delete_all (last_wordcomplete_output);
+		    last_wordcomplete_output = NULL;
+		}
+		word_part_canonical = g_utf8_normalize (word_part, -1, G_NORMALIZE_DEFAULT);
+		cp = strstr (LabelText, word_part_canonical); 
 		/* sanity check: is our word_part found in our label? */
 		if (cp) {
-			gint offset = strlen (word_part); /* in bytes, not chars */
-			if (last_wordcomplete_output) gok_output_delete_all (last_wordcomplete_output);
+			gint offset = strlen (word_part_canonical); /* in bytes, not chars */
 			last_wordcomplete_output = gok_output_new (OUTPUT_KEYSTRING, cp + offset, SPI_KEY_STRING);
 		}
 		else {
 			/* if not then we are doing a 'replace', handle that via EditableText. */
 			g_warning ("Sorry, string replacement is not yet implemented.");
 		}
+		g_free (word_part_canonical);
 	}
 
 	return last_wordcomplete_output;
