@@ -103,15 +103,19 @@ gboolean gok_page_accessmethod_initialize (GtkWidget* pWindowSettings)
 	m_bUsChangingName = TRUE;
 	gtk_combo_set_popdown_strings (GTK_COMBO (pComboBox), items);
 	m_bUsChangingName = FALSE;
-	g_list_free (items);
 
 	/* display the current access method */
 	pEntryAccessMethodName = lookup_widget (pWindowSettings, "entryAccessMethodName");
 	g_assert (pEntryAccessMethodName != NULL);
 	gtk_entry_set_text (GTK_ENTRY (pEntryAccessMethodName), gok_page_accessmethod_get_displayname (m_NameAccessMethod));
 
-	gok_page_accessmethod_change_controls (m_NameAccessMethod);
-
+	/* force the accessmethod widgets to draw only if the current access method is the first one in the list */
+	if (g_ascii_strcasecmp(m_NameAccessMethod, gok_page_accessmethod_get_name (g_list_nth_data (items, 0))) == 0)
+	{
+		gok_page_accessmethod_method_changed (GTK_EDITABLE (pEntryAccessMethodName));
+	}
+	g_list_free (items);
+	
 	return TRUE;
 }
 
