@@ -32,9 +32,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <libspi/keymasks.h>
-#ifndef  SPI_KEYMASK_MOD4
-  #define  SPI_KEYMASK_MOD4 Mod4Mask
-#endif
 #include "main.h"
 #include "gok-spy.h"
 #include "gok-log.h"
@@ -77,10 +74,6 @@ static SPIBoolean gok_spy_mouse_listener (const AccessibleDeviceEvent *event,
 SPIBoolean gok_spy_key_listener (const AccessibleKeystroke *key,
                                  void *user_data);
 
-static GSList*  gok_spy_append_node (GSList *nodes,
-				     Accessible* pAccessible,
-				     AccessibleNodeFlags flags);
-
 static void gok_spy_modifier_listener(const AccessibleEvent *event,
                                       void *user_data);
 
@@ -417,6 +410,8 @@ void gok_spy_close(void)
 		m_gokSpyOpen = FALSE;
 		
 		gok_spy_stop ();
+
+		gok_spy_free_nodes (_priv_ui_nodes);
 		
 		if (m_ptheWindowAccessible != NULL) gok_spy_accessible_unref(m_ptheWindowAccessible);
 		if (m_ptheAccessibleWithText != NULL) gok_spy_accessible_unref(m_ptheAccessibleWithText);
@@ -439,8 +434,8 @@ void gok_spy_close(void)
 		g_queue_free(eventQueue);
 		eventQueue = NULL;
 	}
-	gok_spy_free_nodes (_priv_ui_nodes);
 
+	gok_log ("Exiting SPI mainloop.\n");
 	SPI_exit ();
 
 	gok_log_leave ();
@@ -560,6 +555,7 @@ gok_spy_node_match (AccessibleNode *node, GokSpySearchType type)
 			return TRUE;
 		break;
 	case GOK_SPY_SEARCH_CHILDREN:
+	case GOK_SPY_SEARCH_COMBO:
 	case GOK_SPY_SEARCH_LISTITEMS:
 	case GOK_SPY_SEARCH_ALL:
 		return TRUE;
@@ -574,6 +570,49 @@ gok_spy_node_match (AccessibleNode *node, GokSpySearchType type)
 	return FALSE;
 }
 
+/**
+ *
+ * gok_spy_get_editable:
+ *
+ * Returns the first editable-text child of @search_root.
+ * Used in particular to locate the text-entry child of editable comboboxes.
+ **/
+Accessible *
+gok_spy_get_editable (Accessible *search_root)
+{
+    Accessible *child = NULL;
+
+    /* TODO: complete the logic below, to search the whole tree from the root, 
+       not just the immediate children */
+    gok_spy_accessible_ref (search_root);
+    while (search_root) 
+    {
+	int i = 0;
+	int child_count = Accessible_getChildCount (search_root);
+	while (i < child_count) 
+	{
+	    AccessibleRole role = SPI_ROLE_INVALID;
+	    child = Accessible_getChildAtIndex (search_root, i);
+	    if (child) 
+	    {
+		gok_spy_accessible_implicit_ref (child);
+		if (Accessible_isEditableText (child)) 
+		{
+		    return child;
+		}
+		else 
+		{
+		    gok_spy_accessible_unref (child);
+		}
+	    }
+	    ++i;
+	}
+	gok_spy_accessible_unref (search_root);
+	search_root = NULL;
+    }
+    return NULL;
+}
+
 /**
  *
  * gok_spy_get_list_parent:
@@ -1160,6 +1199,7 @@ GSList* gok_spy_append_node (GSList* nodes,
 	int i = 0;
 	int j = 0;
 	int ntargets = 0;
+	int maxloops = 100;  /* used to gaurd against infinite loops */
 
 	gok_log_enter();
 
@@ -1174,6 +1214,7 @@ GSList* gok_spy_append_node (GSList* nodes,
 		AccessibleHypertext *hypertext = Accessible_getHypertext (pAccessible);
 		AccessibleText *text = Accessible_getText (pAccessible);
 		gint i, nlinks = AccessibleHypertext_getNLinks (hypertext);
+		gok_log ("object contains hypertext.");
 		for (i = 0; i < nlinks; ++i) {
 			AccessibleHyperlink *link = AccessibleHypertext_getLink (hypertext, i);
 			Accessible *link_object = AccessibleHyperlink_getObject (link, 0);
@@ -1241,20 +1282,22 @@ GSList* gok_spy_append_node (GSList* nodes,
 		if (pName == NULL) {
 			Accessible *parent = Accessible_getParent (pAccessible);
 			/* if there are no siblings, we can use an ancestor's name */
+			gok_log("checking ancestor names.");
+			maxloops = 5; /* ignore great great great great grandparent */
 			while (parent && Accessible_getChildCount (parent) == 1) 
 			{
+				char *tmp;
+				if (maxloops-- < 1) { break; }  /* guard against erroneous cyclic parent/child relations */
 				Accessible *tmp_parent = parent;
-				pName = Accessible_getName (tmp_parent); 
-				if (pName != NULL) 
+				tmp = Accessible_getName (tmp_parent); 
+				if (tmp != NULL) 
 				{
-				    char *tmp = NULL;
-				    if (strlen (pName) > 0) 
+				    if (strlen (tmp) > 0) 
 				    {
-					tmp = g_strdup (pName);
+					pName = g_strdup (tmp);
+				    SPI_freeString (tmp);
 					break;
 				    }
-				    SPI_freeString (pName);
-				    pName = tmp;
 				}
 				parent = Accessible_getParent (tmp_parent);
 				Accessible_unref (tmp_parent);
@@ -1267,7 +1310,7 @@ GSList* gok_spy_append_node (GSList* nodes,
 			s = Accessible_getDescription (pAccessible);
 			if (s != NULL) {
 				gint len = strlen (s);
-				if ( len != 0 )  {
+				if ( len > 0 )  {
 					if (len > 18) 
 					{
 						gchar *tmp = NULL;
@@ -1324,10 +1367,10 @@ GSList* gok_spy_append_node (GSList* nodes,
 					else {
 						len = 0;
 					}
-					if (len) {
+					if (len > 0) {
 						if (gs) {
 							tmp = gs;
-							gs = g_strconcat (gs, word, NULL);
+							gs = g_strconcat (tmp, word, NULL);
 							g_free (tmp);
 						}
 						else {
@@ -1393,8 +1436,8 @@ void
 gok_spy_keymap_listener (void)
 {
 	EventNode* en; 
-	gok_log_enter();
 	GList *head = NULL;
+	gok_log_enter();
 
 	/* 
 	 * don't push another keymap event on the queue,
@@ -1796,16 +1839,19 @@ gok_spy_get_toolbar_item (Accessible *toolbar_child)
 	 * ROLE_TOOLBAR containers can have ROLE_PANEL or similarly useless children;
 	 * we need to drill down to the useful thingy i.e. pushbutton, etc.
 	 */
-	Accessible *parent = toolbar_child;
-	Accessible_ref (parent);
-	while (parent) 
+	Accessible *child = toolbar_child, *tmp;
+	Accessible_ref (child);
+	while (child) 
 	{
-		Accessible *child = Accessible_getChildAtIndex (parent, 0);
-		Accessible_unref (parent);
-		if (child && Accessible_isAction (child)) 
+		if (Accessible_isAction (child)) 
+		{
 			return child;
-		else 
-			parent = child;
+		}
+		else {
+			tmp = child;
+			child = Accessible_getChildAtIndex (child, 0);
+			Accessible_unref (tmp);
+		}
 	}
 	return NULL;
 }
@@ -1856,7 +1902,11 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 				if (keyboard_ui_flags.data.gui) 
 					flags.data.is_ui = TRUE; /* toolbar items are ui elements too */
 				ui_flags.data.toolbars = flags.data.is_toolbar_item = TRUE;
-				_priv_ui_nodes = gok_spy_append_node (_priv_ui_nodes, toolbar_item, flags);
+				if (toolbar_item) 
+				{
+					_priv_ui_nodes = gok_spy_append_node (_priv_ui_nodes, toolbar_item, flags);
+					Accessible_unref (toolbar_item);
+				}
 			}
 			else if (keyboard_ui_flags.data.gui && !flags.data.is_menu && gok_spy_is_ui (child, role))
 			{
