@@ -45,6 +45,7 @@ typedef enum {
 	GOK_SPY_SEARCH_APPLICATIONS,
 	GOK_SPY_SEARCH_CHILDREN,
 	GOK_SPY_SEARCH_LISTITEMS,
+	GOK_SPY_SEARCH_COMBO,
 	GOK_SPY_SEARCH_ALL
 } GokSpySearchType;
 
@@ -89,10 +90,13 @@ gboolean gok_spy_is_menu_role (AccessibleRole role);
 Accessible*     gok_spy_get_active_frame (void );
 GSList*         gok_spy_get_list (Accessible* paccessible); 
 Accessible*     gok_spy_get_list_parent (Accessible* paccessible);
+Accessible*     gok_spy_get_editable (Accessible* paccessible);
 GSList*         gok_spy_get_children (Accessible* paccessible); 
 GokSpyUIFlags   gok_spy_update_component_list (Accessible* accessible, GokSpyUIFlags keyboard_ui_flags);
 AccessibleNode* gok_spy_get_ui_list (Accessible* paccessible); 
 AccessibleNode* gok_spy_refresh (AccessibleNode* plist); 
+GSList*         gok_spy_append_node (GSList *nodes, Accessible* pAccessible, AccessibleNodeFlags flags);
+
 void            gok_spy_free (GSList *nodes); /* call this to free up an uneeded list's memory */ 
 
 Accessible*     gok_spy_get_accessibleWithText(void);
