@@ -85,6 +85,8 @@
 #define GCONF_ACCESSIBILITY_KEY "/desktop/gnome/interface/accessibility"
 #define KEYBOARD_ACCESSIBILITY_ENABLE_KEY "/desktop/gnome/accessibility/keyboard/enable"
 
+static gboolean respawn_on_segv = TRUE;
+
 /* pointer to the first keyboard in the list of keyboards */
 static GokKeyboard* m_pKeyboardFirst = NULL;
 
@@ -230,7 +232,15 @@ static int gok_sig_handler(int sig)
 	switch (sig) {
 	case SIGSEGV:
 		segfaults++;
-		fprintf (stderr, "gok: Segmentation fault\n");
+		fprintf (stderr, "gok: Critical (nonrecoverable) error.\n");
+		/* N.B.: if SPI_exit SEGVs, GOK will keep restarting until killed by SIGTERM */
+#if ! defined ENABLE_LOGGING_NORMAL
+		if (respawn_on_segv)
+		{
+		    fprintf (stderr, "Restarting GOK.\n"); 
+		    g_spawn_command_line_async ("gok", NULL); 
+		}
+#endif
 		break;
 	case SIGTERM:
 		fprintf (stderr, "gok: exiting (terminated)\n");
@@ -240,9 +250,6 @@ static int gok_sig_handler(int sig)
 		break;
 	}
 	
-	if (segfaults == 1)
-		gok_main_close();
-
 	_exit(1);
 	return -1;  /* this line gets rid of compiler warning */ 
 }
@@ -819,7 +826,7 @@ void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long ti
 		if (gok_data_get_drive_corepointer ()) {
 			XWarpPointer (display, None, 
 				      GDK_WINDOW_XWINDOW (root),
-				      0, 0, 0, 0, dx, dy);
+				      0, 0, 0, 0, motion_data[0], motion_data[1]);
 		}
 	}
 }
@@ -1550,6 +1557,18 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 		pKeyboardPrevious = pKeyboard;
 	}
 
+	/* create the Alpha and Alpha-Freq keyboards */
+	pKeyboard = gok_keyboard_get_alpha ();
+	pKeyboard->pKeyboardNext = pKeyboardPrevious->pKeyboardNext;
+	pKeyboardPrevious->pKeyboardNext = pKeyboard;
+	pKeyboard->pKeyboardPrevious = pKeyboardPrevious;
+	pKeyboardPrevious = pKeyboard;
+	pKeyboard = gok_keyboard_get_alpha_by_frequency ();
+	pKeyboard->pKeyboardNext = pKeyboardPrevious->pKeyboardNext;
+	pKeyboardPrevious->pKeyboardNext = pKeyboard;
+	pKeyboard->pKeyboardPrevious = pKeyboardPrevious;
+	pKeyboardPrevious = pKeyboard;	
+
 	/* look at each file in the directory */
 	while ((pDirectoryEntry = readdir (pDirectoryKeyboards)) != NULL)
 	{
@@ -2019,7 +2038,14 @@ gok_main_close()
 
 	/* note, call all closing functions that delete UI before calling gok_settingsdialog_close */
 	gok_scanner_close(); /* deletes UI */
-	gok_settingsdialog_close();
+
+	/*	gok_settingsdialog_close(); */
+/**
+ * the above, in context of the following calls,  is fundamentally broken, in that it 
+ * destroys widgets which are either already destroyed, or are about to be destroyed below. 
+ * Anyhow, any memory it uses will be freed on exit.   Better not to call it until the 
+ * whole UI cleanup code can get fixed. - wph
+ **/
 	
 	gok_data_close();
 
@@ -2243,7 +2269,7 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 								 GTK_DIALOG_DESTROY_WITH_PARENT,
 								 GTK_MESSAGE_INFO,
 								 GTK_BUTTONS_OK,
-								 "GOK has enabled Sticky Keys, which it requires.\n");
+								 _("GOK has enabled Sticky Keys, which it requires.\n"));
 				
 				  /* Destroy the dialog when the user responds to it (e.g. clicks a button) */
 				  g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
@@ -2264,7 +2290,7 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
-						 "GOK cannot run because XKB display extension is missing.\n");		
+						 _("GOK cannot run because XKB display extension is missing.\n"));
 			g_signal_connect_swapped (GTK_OBJECT (dialog), "response",
 					  G_CALLBACK (gtk_widget_destroy),
 					  GTK_OBJECT (dialog));
